// Ionic Starter App

// angular.module is a global place for creating, registering and retrieving Angular modules
// 'starter' is the name of this angular module example (also set in a <body> attribute in index.html)
// the 2nd parameter is an array of 'requires'
// 'starter.services' is found in services.js
// 'starter.controllers' is found in controllers.js
var newsApp = angular.module('starter', ['ionic', 'starter.services', 'starter.controllers', 'angulartics', 'angulartics.google.analytics'])



.config(function($stateProvider, $urlRouterProvider, $analyticsProvider) {

  // Ionic uses AngularUI Router which uses the concept of states
  // Learn more here: https://github.com/angular-ui/ui-router
  // Set up the various states which the app can be in.
  // Each state's controller can be found in controllers.js
  $stateProvider

    // setup an abstract state for the tabs directive
    .state('tab', {
      url: "/tab",
      abstract: true,
      templateUrl: "templates/tabs.html"
    })

    // the pet tab has its own child nav-view and history
    .state('tab.news-index', {
      url: '/main',
      views: {
        'main': {
          templateUrl: 'templates/app-index.html',
          controller: 'MainController'
        }
      }
    })

    // the pet tab has its own child nav-view and history
    .state('tab.news-topic', {
      url: '/news-topic/:topicID',
      views: {
        'main': {
          templateUrl: 'templates/news-topic.html',
          controller: 'NewsTopicController'
        }
      }
    })

    // the pet tab has its own child nav-view and history
    .state('tab.news-content', {
      url: '/news-content/:articleID',
      views: {
        'main': {
          templateUrl: 'templates/news-content.html',
          controller: 'ContentController'
        }
      }
    })

    .state('tab.pet-detail', {
      url: '/article/:articleID',
      views: {
        'pets-tab': {
          templateUrl: 'templates/pet-detail.html',
          controller: 'PetDetailCtrl'
        }
      }
    })

    .state('tab.setup', {
      url: '/setup',
      views: {
        'setting-tab': {
          templateUrl: 'templates/setup.html',
          controller: 'SetupController'
        }
      }
    })

    .state('tab.setting', {
      url: '/setting',
      views: {
        'setting-tab': {
          templateUrl: 'templates/setting.html',
          controller: 'SettingController'
        }
      }
    })

    // the pet tab has its own child nav-view and history
    .state('tab.bookmark', {
      url: '/bookmark',
      views: {
        'bookmark-tab': {
          templateUrl: 'templates/bookmark.html',
          controller: 'BookmarkController'
        }
      }
    })

    .state('tab.adopt', {
      url: '/adopt',
      views: {
        'adopt-tab': {
          templateUrl: 'templates/adopt.html'
        }
      }
    })

    .state('tab.about', {
      url: '/about',
      views: {
        'about-tab': {
          templateUrl: 'templates/about.html'
        }
      }
    });

  // if none of the above states are matched, use this as the fallback
  $urlRouterProvider.otherwise('/tab/main');

});


//Add this to have access to a global variable
newsApp.run(function ($rootScope, $state, $ionicLoading) {
    //$rootScope.serverLocation = 'http://localhost:3000'; //global variable
    $rootScope.serverLocation = 'http://newscast.io'; //global variable

	//show loading icon
	$rootScope.showLoading = function() {
	  $rootScope.loading = $ionicLoading.show({
		content: 'Loading',
	  });
	};


	$rootScope.hideLoading = function(){
	  //$rootScope.loading.hide();
	  $ionicLoading.hide();
	};				


	
	//DB Setup
	var db = openDatabase('castDB', '1.0.0', 'Cast DB', 2 * 1024 * 1024);
	db.transaction(function (tx) {  
	   tx.executeSql('CREATE TABLE IF NOT EXISTS subscription (topic_id)');
	   tx.executeSql('CREATE TABLE IF NOT EXISTS bookmark (article_id, title, source, image_url)');
	});
	
	db.transaction(function (tx) {
	  tx.executeSql('SELECT * FROM subscription', [], function (tx, results) {
	  	var len = results.rows.length, i;
		if(len == 0 )
		{
			console.log('empty database');
			
			
			//Switch page
			$state.go('tab.setup');
		}
		
	 }, null);
	});

	
	
});


//jQuery Function
//valueList
(function( $ ){
      $.fn.valueList = function(){
            return $.map( this, function (elem) {
                  return elem.value || "";
            }).join( "," );
      };
})( jQuery );
